SystemOrganization addCategory: #'Rest-Server-Handlers'!
SystemOrganization addCategory: #'Rest-Server-Services'!
SystemOrganization addCategory: #'Rest-Server'!

Object subclass: #RestHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestHttpHandler methodsFor: 'as yet unclassified' stamp: 'cs 3/18/2009 15:26'!
isAuthorized: anHeaderField

	^ self subclassResponsibility! !

!RestHttpHandler methodsFor: 'authentication' stamp: 'topa 3/5/2009 11:29'!
needsAuthentication
	"override, if HTTPAuth is needed"
	
	^ false! !

!RestHttpHandler methodsFor: 'request processing' stamp: 'SR 3/9/2009 15:52'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	self needsAuthentication 
		ifTrue: [((anHTTPRequest headers includesFieldNamed: 'Authorization') and: 
			[self isAuthorized: (anHTTPRequest headers fieldNamed: 'Authorization')])
				ifFalse: [^ HTTPResponse unauthorized]].
	
	^ nil! !

RestHttpHandler subclass: #RestResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResourceHttpHandler class methodsFor: 'error handling' stamp: 'cs 3/18/2009 15:29'!
responseForFailure

	^ HTTPResponse internalServerError! !

!RestResourceHttpHandler methodsFor: 'parsing' stamp: 'cs 3/18/2009 15:26'!
pathComponents: anHTTPRequest
	"Return a list with all components of the path in the HTTP request. Remove a leading '/'."
	
	self flag: #TODO. "What about prefixes here?"
	^ anHTTPRequest uri identifierPath allButFirst! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'topa 4/2/2009 17:44'!
process: anHTTPRequest 

	^ (super process: anHTTPRequest) ifNil: [
		[(self processorForRequest: anHTTPRequest) 
			process: (self rootComponent: anHTTPRequest ifAbsent: [^ self class responseForFailure])
			components: (self pathComponents: anHTTPRequest) allButFirst] 
				on: RestError
				do: [:error |error asHTTPResponse]]! !

!RestResourceHttpHandler methodsFor: 'as yet unclassified' stamp: 'topa 3/24/2009 15:32'!
processorForRequest: anHTTPRequest

	| processorClass |
	anHTTPRequest isGet ifTrue: [processorClass := RestGetRequestProcessor].
	anHTTPRequest isPost ifTrue: [processorClass := RestPostRequestProcessor].
	anHTTPRequest isDelete ifTrue: [processorClass := RestDeleteRequestProcessor].
	anHTTPRequest isPut ifTrue: [processorClass := RestPutRequestProcessor].

	^ processorClass "dispatch if not found"
		ifNil: [RestNotImplementedError signal: 'This method is not implemented by Rest']
		ifNotNil: [processorClass newFrom: anHTTPRequest handler: self]! !

!RestResourceHttpHandler methodsFor: 'as yet unclassified' stamp: 'mfp 3/6/2009 11:29'!
requestAllowed: aRequest for: anObject
	^ true! !

!RestResourceHttpHandler methodsFor: 'parsing' stamp: 'mt 3/2/2009 15:03'!
rootComponent: anHTTPRequest ifAbsent: aBlock
	"Return an association between the resource name and the resource class for the root component according to the HTTP request."

	^ self class rootResources
		associationAt: ((self pathComponents: anHTTPRequest) at: 1 ifAbsent: aBlock)
		ifAbsent: aBlock! !

!RestResourceHttpHandler methodsFor: 'as yet unclassified' stamp: 'mig 3/25/2009 16:41'!
selectorAllowed: aSelector for: anObject

	^ (anObject class allSelectors includes: aSelector)! !

Object subclass: #RestRequestProcessor
	instanceVariableNames: 'request handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

RestRequestProcessor subclass: #RestDeleteRequestProcessor
	instanceVariableNames: 'postData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestDeleteRequestProcessor methodsFor: 'processing' stamp: 'SR 3/20/2009 13:51'!
responseBodyForRoot: root components: components
	"Do recursive entity resolving. Start with root. Return an empty body due to DELETE request."
	
	| entity parentEntity selector |
	entity := self resolveId: root value all components: components.
	self request requestLine requestURI asString = entity restUri
		ifTrue: [
			(entity respondsTo: #delete)
				ifTrue: [entity perform: #delete]
				ifFalse: [RestResolveError signal: 'Selector #delete not found.']]
		ifFalse: [
			parentEntity := self resolveId: root value all components: components allButLast allButLast.
			selector := self selectorFor: components allButLast last.
			(parentEntity perform: selector with: entity)].
	^ String new! !

!RestDeleteRequestProcessor methodsFor: 'as yet unclassified' stamp: 'mt 4/16/2009 15:45'!
selectorFor: resourceName
	"Return the selector for adding a resource depending on the given resource name."
	
	| selector |
	selector := 'detach', resourceName capitalized.
	(selector endsWith: 'ies') ifTrue: [selector := (selector allButLast: 3), 'y'].
	(selector endsWith: 's') ifTrue: [selector := selector allButLast].
	selector := (selector, ':') asSymbol.
	^ selector! !

RestRequestProcessor subclass: #RestGetRequestProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'SR 4/17/2009 08:46'!
resolveMethod: anEntity components: pathComponents
	
	| selector resource |
	pathComponents ifEmpty: [^ anEntity].
	
	selector := (pathComponents first, 'WithDeleted')  asSymbol.
	(self handler selectorAllowed: selector for: anEntity)  
		ifTrue: [resource := anEntity perform: selector]
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].
	
	self handler requestAllowed: self request for: resource.

	^ (pathComponents size = 2 and: [pathComponents last = 'all'])
		ifTrue: [resource]
		ifFalse: [self resolveId: resource components: pathComponents allButFirst]! !

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'SR 4/17/2009 08:51'!
responseBodyForRoot: root components: pathComponents
	"Do recursive entity resolving. Start with root."
		
	| entity |
	pathComponents isEmpty
		ifTrue: [entity := self resolveId: root value all components: pathComponents]
		ifFalse: [entity := self resolveId: root value allWithDeleted components: pathComponents].

	^ Json render: (entity isCollection
		ifTrue: [entity collect: [:each | each restUri]]
		ifFalse: [entity])! !

RestRequestProcessor subclass: #RestPostRequestProcessor
	instanceVariableNames: 'postData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPostRequestProcessor methodsFor: 'adding' stamp: 'mt 3/2/2009 15:50'!
addResource: aResourceName to: anEntity
	"Resolve the selector and perform an add-operation on the given entity with the postData."
	
	| selector |
	selector := self selectorFor: aResourceName.
	
	(anEntity respondsTo: selector)
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].
		
	^ (anEntity perform: selector with: self postData) restUri
! !

!RestPostRequestProcessor methodsFor: 'private' stamp: 'mig 3/13/2009 12:24'!
postData
	"Data could be JSON string with properties or a simple string with a rest uri (not JSON). 
	The first one will be parsed into a dictionary. The last one will be resolved to the real entity."

	postData ifNil: [ | data |
		self request class category = 'Hyper' ifFalse: [self request ensureFullRead].
		data := Json readFrom: self request entityBody value readStream.
		postData := data isString 
			ifTrue: [self resolveUri: data]
			ifFalse: [self parseData: data].
	].
	^ postData! !

!RestPostRequestProcessor methodsFor: 'resolving' stamp: 'mfp 3/6/2009 14:49'!
resolveMethod: anEntity components: pathComponents
	"New recursion abort condition. Perform add-operation due to POST request (instead of returning the entity)."
	
	^ pathComponents size = 1
		ifTrue: [
			self handler requestAllowed: self request for: anEntity.
			self addResource: pathComponents first to: anEntity]
		ifFalse: [
			super resolveMethod: anEntity components: pathComponents]! !

!RestPostRequestProcessor methodsFor: 'processing' stamp: 'mt 3/2/2009 15:51'!
responseBodyForRoot: root components: components
	"Do recursive entity resolving. Start with root."
	
	^ components isEmpty
		ifTrue: [self addResource: root key to: root value]
		ifFalse: [self resolveId: root value all components: components]! !

!RestPostRequestProcessor methodsFor: 'adding' stamp: 'mt 4/16/2009 14:56'!
selectorFor: resourceName
	"Return the selector for adding a resource depending on the given resource name."
	
	| selector |
	resourceName capitalized = 'Developers'
		ifTrue: [selector := 'attachDeveloper']
		ifFalse: [selector := self postData isCollection
			ifTrue: ['addNew', resourceName capitalized]
			ifFalse: ['add', resourceName capitalized]].
	(selector endsWith: 'ies') ifTrue: [selector := (selector allButLast: 3), 'y'].
	(selector endsWith: 's') ifTrue: [selector := selector allButLast].
	selector := (selector, ':') asSymbol.
	^ selector! !

RestRequestProcessor subclass: #RestPutRequestProcessor
	instanceVariableNames: 'putData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPutRequestProcessor methodsFor: 'private' stamp: 'mt 3/2/2009 16:00'!
putData

	putData ifNil: [
		putData := Json readFrom: (HTTPString stringFromBytes: self request putData)
			value "convert bytes to string"
			readStream "convert string to readstream"].
	^ putData! !

!RestPutRequestProcessor methodsFor: 'processing' stamp: 'mt 3/16/2009 13:01'!
responseBodyForRoot: root components: pathComponents
	"Do recursive entity resolving. Start with root."
	
	| entity selector |
	entity := self resolveId: root value all components: pathComponents.
	(self parseData: self putData) keysAndValuesDo: [:key :value | 
		selector := key asSymbol asMutator.
		(((entity class primitivePropertyNames union: entity class referencePropertyNames) includes: key asSymbol) and:
			[entity respondsTo: selector])
				ifTrue: [entity perform: selector with: value]
				ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})]].
	^ entity restUri! !

!RestRequestProcessor class methodsFor: 'instance creation' stamp: 'cs 2/23/2009 18:06'!
newFrom: anHTTPRequest handler: aHandler
	^ self new
		request: anHTTPRequest;
		handler: aHandler! !

!RestRequestProcessor methodsFor: 'private' stamp: 'SR 3/25/2009 17:49'!
beginTransaction

	^ (Smalltalk includesKey: #System) 
		ifTrue: [(Smalltalk at: #System) beginTransaction]
		ifFalse: [true]! !

!RestRequestProcessor methodsFor: 'private' stamp: 'SR 3/25/2009 17:50'!
commitTransaction

	^ (Smalltalk includesKey: #System) 
		ifTrue: [(Smalltalk at: #System) commitTransaction]
		ifFalse: [true]! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
handler

	^ handler! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
handler: anObject

	handler := anObject! !

!RestRequestProcessor methodsFor: 'helping' stamp: 'cs 3/18/2009 15:26'!
parseData: aDictionary

	| newDict |
	newDict := Dictionary new.
	aDictionary keysAndValuesDo: [:key :value | newDict add: (
		(key startsWith: 'ref ')
			ifTrue: [(key allButFirst: 4) -> (value ifNotNil: [self resolveUri: value])]
			ifFalse: [key -> value])].
	^ newDict! !

!RestRequestProcessor methodsFor: 'processing' stamp: 'SR 3/25/2009 17:55'!
process: root components: pathComponents
	"The root should be an association between name and class of the root resource."

	| result |
	self beginTransaction.
	result := self responseBodyForRoot: root components: pathComponents.
	self commitTransaction
		ifTrue: [^ HTTPResponse ok entity: result]
		ifFalse: [^ HTTPResponse internalServerError].! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:57'!
request

	^ request! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'topa 3/3/2009 20:58'!
request: anObject

	request := anObject! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'SR 3/9/2009 13:11'!
resolveId: aCollection components: pathComponents
	"Try to resolve the next element in the path to a number (id). This id should be a valid index in the given collection. Hint: Recursive."

	| id resource |
	pathComponents ifEmpty: [^ aCollection].
	
	id := pathComponents first asNumber.
	resource := aCollection 
		detect: [:each | each id = id] 
		ifNone: [RestResolveError signal: ('Entity with id {1} not found.' format: {id})].

	self handler requestAllowed: self request for: resource.

	^ self resolveMethod: resource components: pathComponents allButFirst! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'mig 3/25/2009 16:38'!
resolveMethod: anEntity components: pathComponents
	"Try to resolve the next element in the path to a method. This method/message should be understood by the entity. Hint: Recursive."
	
	| selector resource |
	pathComponents ifEmpty: [^ anEntity].
	
	selector := pathComponents first asSymbol.
	(self handler selectorAllowed: selector for: anEntity)  
		ifTrue: [resource := anEntity perform: selector]
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].
	
	self handler requestAllowed: self request for: resource.

	^ self resolveId: resource components: pathComponents allButFirst! !

!RestRequestProcessor methodsFor: 'helping' stamp: 'mfp 3/6/2009 15:53'!
resolveUri: aString
	"Finds an object with the provided uri by using an pseudo-get-request. This is the only reason why we need a reference to the handler."

	|component components|
	components := aString findTokens: '/'.
	component :=  self handler class rootResources 
		at: components first 
		ifAbsent:[RestResolveError signal: 'invalid URI given'].

	^ self 
		resolveId: component all
		components: components allButFirst! !

!RestRequestProcessor methodsFor: 'processing' stamp: 'mt 3/2/2009 15:14'!
responseBodyForRoot: root components: pathComponents

	self subclassResponsibility.! !

!RestRequestProcessor methodsFor: 'private' stamp: 'SR 3/25/2009 17:53'!
transactionConflicts

	^ (Smalltalk includesKey: #System) 
		ifTrue: [(Smalltalk at: #System) transactionConflicts at: #commitResult]
		ifFalse: [#success]! !

!HTTPPut methodsFor: '*rest-server' stamp: 'topa 4/3/2009 17:02'!
for: aRequestLine readFrom: aSwazooStream 

	super for: aRequestLine readFrom: aSwazooStream.
	self octetDataFrom: aSwazooStream.! !

Resource subclass: #RestHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Services'!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService commentStamp: 'mt 2/16/2009 14:56' prior: 0!
I am a restful http service. I am primarily used for the XP integration platform project. (Package: 'Xpf')!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService class methodsFor: 'testing' stamp: 'topa 3/3/2009 21:11'!
isRunning

	^ self site notNil and: [site isServing]! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:11'!
port

	^ self site port! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
resource

	^ resource! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
resource: anObject

	resource := anObject.! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:12'!
site

	^ site! !

!RestHttpService class methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:13'!
site: anObject

	site := anObject.! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 16:08'!
start

	self startOn: 8082.

! !

!RestHttpService class methodsFor: 'service' stamp: 'topa 3/3/2009 21:14'!
startOn: aPort
	"Start exactly one http service. Does nothing if server is already running."

	self isRunning ifTrue: [^ self].
	self site: (Site new
		name: 'rest';
		host: '*' port: aPort).

	SwazooServer singleton addSite: self site.
	self resource: (RestHttpService new uriPattern: '/').
	self site addResource: self resource.
	SwazooServer singleton startSite: 'rest'.! !

!RestHttpService class methodsFor: 'service' stamp: 'topa 3/3/2009 21:15'!
stop
	"Shutdown the server. Does nothing if server is not running."

	self site ifNotNil:
		[self site stop.
		SwazooServer singleton removeSite: self site.
		self site: nil; resource: nil].! !

!RestHttpService methodsFor: 'serving' stamp: 'topa 2/23/2009 17:48'!
answerTo: request
	
	^ (handlers
		detect: [:ass| request uri asString startsWith: ass key]
		ifNone: [^ HTTPResponse notFound])
			value new process: request! !

!RestHttpService methodsFor: 'initialization' stamp: 'topa 3/3/2009 21:10'!
initialize

	super initialize.
	handlers := OrderedCollection new.	"needed for priorities"! !

!RestHttpService methodsFor: 'serving' stamp: 'topa 3/3/2009 21:09'!
installHandler: aRequestHandlerClass on: aPathString

	(handlers anySatisfy: [:ass | ass key = aPathString ]) ifTrue: [^ false].
	handlers add: aPathString -> aRequestHandlerClass.! !

!Collection methodsFor: '*rest-server' stamp: 'topa 2/25/2009 13:25'!
jsonWriteOn: aStream

	aStream nextPut: $[.
	self
		do: [ :each | each jsonWriteOn: aStream]
		separatedBy: [aStream nextPut: $,].
	aStream nextPut: $].! !

Error subclass: #RestError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

RestError subclass: #RestNotAllowedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestNotAllowedError methodsFor: 'as yet unclassified' stamp: 'mfp 3/6/2009 11:42'!
asHTTPResponse

	^ HTTPResponse forbidden entity: self messageText! !

RestError subclass: #RestNotImplementedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestNotImplementedError methodsFor: 'as yet unclassified' stamp: 'topa 3/24/2009 15:29'!
asHTTPResponse

	^ HTTPResponse notImplemented entity: self messageText! !

RestError subclass: #RestResolveError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResolveError methodsFor: 'as yet unclassified' stamp: 'topa 3/3/2009 21:02'!
asHTTPResponse

	^ HTTPResponse notFound entity: self messageText! !
